use crate::deserialize::*;
use crate::IconID;
use serde::{Deserialize, Serialize};
use std::collections::HashMap;

#[derive(Clone, Debug, Deserialize, Serialize)]
struct FavIconRequest {
    #[serde(deserialize_with = "to_u64")]
    pub id: IconID,
    pub data: String,
}

#[derive(Clone, Debug, Deserialize, Serialize)]
pub struct FavIcon {
    pub mime_type: String,
    pub data: String,
}

impl FavIcon {
    /// destroy this entry and gain ownership of all the data it contains
    pub fn decompose(self) -> (String, String) {
        (self.mime_type, self.data)
    }
}

#[derive(Clone, Debug, Deserialize)]
pub struct FavIcons {
    #[serde(deserialize_with = "to_i64")]
    pub last_refreshed_on_time: i64,
    #[serde(with = "favicons")]
    pub favicons: HashMap<IconID, FavIcon>,
}

mod favicons {
    use super::{FavIcon, FavIconRequest};

    use std::collections::HashMap;

    use serde::de::{Deserialize, Deserializer};

    pub fn deserialize<'de, D>(deserializer: D) -> Result<HashMap<u64, FavIcon>, D::Error>
    where
        D: Deserializer<'de>,
    {
        let mut map = HashMap::new();
        for item in Vec::<FavIconRequest>::deserialize(deserializer)? {
            let tmp: Vec<&str> = item.data.split(';').collect();
            map.insert(
                item.id,
                FavIcon {
                    mime_type: tmp[0].to_string(),
                    data: tmp[1].to_string(),
                },
            );
        }
        Ok(map)
    }
}
