use crate::deserialize::*;
use crate::models::feedgroup::FeedsGroups;
use crate::FeedID;
use crate::IconID;
use serde::Deserialize;

#[derive(Clone, Debug, Deserialize)]
pub struct Feed {
    #[serde(deserialize_with = "to_u64")]
    pub id: FeedID,
    #[serde(deserialize_with = "to_u64")]
    pub favicon_id: IconID,
    pub title: String,
    pub url: String,
    pub site_url: Option<String>,
    #[serde(deserialize_with = "bool_from_number")]
    pub is_spark: bool,
    #[serde(deserialize_with = "to_i64")]
    pub last_updated_on_time: i64,
}

#[derive(Clone, Debug, Deserialize)]
pub struct Feeds {
    #[serde(deserialize_with = "to_i64")]
    pub last_refreshed_on_time: i64,
    pub feeds: Vec<Feed>,
    pub feeds_groups: Vec<FeedsGroups>,
}

impl Feed {
    /// destroy this entry and gain ownership of all the data it contains
    pub fn decompose(self) -> (FeedID, IconID, String, String, String, bool, i64) {
        (
            self.id,
            self.favicon_id,
            self.title,
            self.url,
            self.site_url.unwrap_or_else(|| "".to_string()),
            self.is_spark,
            self.last_updated_on_time,
        )
    }
}
