use crate::deserialize::*;
use crate::FeedID;
use crate::ItemID;

use serde::Deserialize;

#[derive(Clone, Debug, Deserialize)]
pub struct Item {
    #[serde(deserialize_with = "to_u64")]
    pub id: ItemID,
    #[serde(deserialize_with = "to_u64")]
    pub feed_id: FeedID,
    pub title: String,
    pub author: String,
    pub html: String,
    pub url: String,
    #[serde(deserialize_with = "bool_from_number")]
    pub is_saved: bool,
    #[serde(deserialize_with = "bool_from_number")]
    pub is_read: bool,
    #[serde(deserialize_with = "to_i64")]
    pub created_on_time: i64,
}

impl Item {
    /// destroy this entry and gain ownership of all the data it contains
    pub fn decompose(
        self,
    ) -> (
        ItemID,
        FeedID,
        String,
        String,
        String,
        String,
        bool,
        bool,
        i64,
    ) {
        (
            self.id,
            self.feed_id,
            self.title,
            self.author,
            self.html,
            self.url,
            self.is_saved,
            self.is_read,
            self.created_on_time,
        )
    }
}

#[derive(Clone, Debug, Deserialize)]
pub struct Items {
    #[serde(deserialize_with = "to_i64")]
    pub last_refreshed_on_time: i64,
    #[serde(deserialize_with = "to_u64")]
    pub total_items: u64,
    pub items: Vec<Item>,
}

impl Items {
    /// destroy this entry batch and gain ownership of all the entries it contains
    pub fn decompose(self) -> (u64, Vec<Item>) {
        (self.total_items, self.items)
    }
}
