use crate::deserialize::*;
use crate::FeedID;
use crate::ItemID;
use crate::LinkID;
use serde::Deserialize;

#[derive(Clone, Debug, Deserialize)]
pub struct Link {
    #[serde(deserialize_with = "to_u64")]
    pub id: LinkID,
    #[serde(deserialize_with = "to_u64")]
    pub feed_id: FeedID,
    #[serde(deserialize_with = "to_u64")]
    pub item_id: ItemID,
    pub temperature: f64,
    #[serde(deserialize_with = "bool_from_number")]
    pub is_item: bool,
    #[serde(deserialize_with = "bool_from_number")]
    pub is_local: bool,
    #[serde(deserialize_with = "bool_from_number")]
    pub is_saved: bool,
    pub title: String,
    pub url: String,
    #[serde(deserialize_with = "vec_ids_from_string")]
    pub item_ids: Vec<ItemID>,
}

#[derive(Clone, Debug, Deserialize)]
pub struct Links {
    #[serde(deserialize_with = "to_i64")]
    pub last_refreshed_on_time: i64,
    pub links: Vec<Link>,
}

impl Link {
    /// destroy this entry and gain ownership of all the data it contains
    pub fn decompose(
        self,
    ) -> (
        LinkID,
        FeedID,
        ItemID,
        f64,
        bool,
        bool,
        bool,
        String,
        String,
        Vec<ItemID>,
    ) {
        (
            self.id,
            self.feed_id,
            self.item_id,
            self.temperature,
            self.is_item,
            self.is_local,
            self.is_saved,
            self.title,
            self.url,
            self.item_ids,
        )
    }
}
